#!/bin/sh

string_in_file() {
  local expr="$1"
  shift
  local fname="$@"
  command grep --quiet --fixed-strings "$expr" $fname
}

regex_in_file() {
  local expr="$1"
  shift
  local fname="$@"
  command grep --quiet --perl-regexp "$expr" $fname
}

replace_if_not_in_file() {
  ## if string $1 is not in any of the files $3 ..., replaces sed-replace pattern $2 in files $3 ...
  local string="$1"
  local expr="$2"
  shift 2
  local fname="$@"
  string_in_file "$string" $fname || sed --in-place --regexp-extended "$expr" $fname
}

replace_if_not_already() {
  ## replaces sed-replace pattern $1 in files $2 ...
  local expr="$1"
  shift
  local fname="$@"
  case "$expr" in s*) ;; *) echo 'Error: replace_if_not_already expects sed-replace (extended) expression as $1' && return 1 ;; esac
  local delim="$(echo "$expr" | sed -r 's,.(.).*,\1,')"  # second character of expr is delimiter
  replacement=$(
    /bin/echo "$expr" | sed -r "s#(\\\\[^$delim])#\\\\\\\\\1#g" |
    while read _1 _2 replacement _3; do echo "$replacement" ; done)  # parse out replacement string
  replacement="$(echo -E "$replacement" | sed 's,\\[0-9],.*?,g')"  # HACK: not deterministic
  regex_in_file "$replacement" $fname ||
    \sed --in-place --regexp-extended "$expr" $fname
}

append_if_not_already() {
  ## appends string $1 to filename $2
  [ "$#" = '2' ] || { echo 'Error: append_if_not_already only accepts two arguments (string and filename)' && return 1; }
  local string="$1"
  local fname="$2"
  string_in_file "$string" $fname || { sed -i -e '$a\' $fname && echo "$string" >> $fname; }
}

postinstall_hook() {
  local name="$1"
  shift
  local hookdir="/etc/live-installer/postinst"
  mkdir -p $hookdir
  rm "$hookdir/$name"
  case "$#" in
    0) cat > "$hookdir/$name" ;;
    1) echo "$1" > "$hookdir/$name" ;;
    *) echo 'Error: postinstall_hook accepts script body as stdin (heredoc) or second argument' && return 1 ;;
  esac
  chmod +x "$hookdir/$name"
}

set_default_application() {
  ## sets xdg-mime default application $1 for ;-separated mime types string $2
  local application="$1"
  local mimetypes="$2"
  local defaults_file="/etc/skel/.local/share/applications/mimeapps.list"
  [ ! -e "$defaults_file" ] &&
    mkdir -p "$(dirname "$defaults_file")" &&
    echo "[Default Applications]" > "$defaults_file"
  [ -e "/usr/share/applications/$application.desktop" ] || { 
    echo "Warning: set_default_application: application '$application' does not have a desktop file" &&
    return;
  }
  for mimetype in $(echo "$mimetypes" | tr ';' ' '); do
    replace_if_not_already "s,^$mimetype=.+,$mimetype=$application.desktop," "$defaults_file"
    append_if_not_already "$mimetype=$application.desktop" "$defaults_file"
  done
}
